#include <RunningMedian.h>


// GPIOs
#define JOY1_GPIO   35
#define JOY2_GPIO   39
#define CUEX_GPIO   36 
#define CURV_GPIO   33 


// Median arrays 
#define JOY1  0
#define JOY2  1
#define CUEX  2
#define CURV  3


// Knob and joystick current positions
float joy1;
float joy2;
float cuex;
float curv;


// Physical positions scale
#define MAXPOSITION   4095
#define MIDPOSITION   (MAXPOSITION*0.6)
#define KNEEPOS       (MAXPOSITION*0.55)
#define MIDPOSOVERLAP (MAXPOSITION*0.2)
#define LOWPOSITION   (MAXPOSITION*0.05)
#define MINPOSITION   0
#define HI_DEADZN     MAXPOSITION
#define LO_DEAZN      MINPOSITION


// Output levels scale 
#define MAXOUT        1000                // Full volume
#define NORMALOUT     (MAXOUT*0.98)
#define BOOST         (MAXOUT-NORMALOUT)
#define LOWOUT        (NORMALOUT*0.75)    // Volume when the joystick is at a specific position (LOWPOSITION) on the lower half of the movenent
#define MINOUT        0                   // Silent



#define ARRAYSIZE 5   // number of samples to get median from
RunningMedian avgarray[4] = { RunningMedian(ARRAYSIZE), RunningMedian(ARRAYSIZE), RunningMedian(ARRAYSIZE), RunningMedian(ARRAYSIZE) }; 




// ------------------------------------------------------------------------------
void readfaders(void) {
  avgarray[JOY1].add(map(constrain(analogRead(JOY1_GPIO), LO_DEAZN, HI_DEADZN), LO_DEAZN, HI_DEADZN, MINPOSITION, MAXPOSITION));
  avgarray[JOY2].add(map(constrain(analogRead(JOY2_GPIO), LO_DEAZN, HI_DEADZN), LO_DEAZN, HI_DEADZN, MINPOSITION, MAXPOSITION));
  avgarray[CUEX].add(map(constrain(analogRead(CUEX_GPIO), LO_DEAZN, HI_DEADZN), LO_DEAZN, HI_DEADZN, MINPOSITION, MAXPOSITION));
  avgarray[CURV].add(map(constrain(analogRead(CURV_GPIO), LO_DEAZN, HI_DEADZN), LO_DEAZN, HI_DEADZN, MINPOSITION, MAXPOSITION));
  
  joy1 = (float)avgarray[JOY1].getMedian()/MAXPOSITION;
  joy2 = (float)avgarray[JOY2].getMedian()/MAXPOSITION;
  cuex = (float)avgarray[CUEX].getMedian()/MAXPOSITION;
  curv = (float)avgarray[CURV].getMedian()/MAXPOSITION;
}



// ------------------------------------------------------------------------------
float xfade (float arg1, float arg2) {

  int result;
  uint16_t stickPos = (uint16_t) (arg1 * MAXPOSITION);
  uint16_t oppPos =   (uint16_t) (arg2 * MAXPOSITION);
  
//  float x = arg1-arg2;
//  float y = arg1+arg2-1;

  float tophalf      = map(constrain((stickPos+oppPos)/2, KNEEPOS, MAXPOSITION), KNEEPOS, MAXPOSITION, 0, MAXPOSITION)/(float)MAXPOSITION;
  float firstquarter = map(constrain((stickPos+oppPos)/2, 0.5*MAXPOSITION, 0.75*MAXPOSITION), 0.5*MAXPOSITION, 0.75*MAXPOSITION, 0, MAXPOSITION)/(float)MAXPOSITION;
  int midposition = MIDPOSITION-MIDPOSOVERLAP + ( MIDPOSOVERLAP * (1.0-curv) * (1-firstquarter) );    // Adjustable overlap over center position of joystick
  

  if ( stickPos >= midposition )                                        // UPPER half
    result = map(stickPos, midposition, MAXPOSITION, NORMALOUT, NORMALOUT);
    result += BOOST * tophalf;
    
  if ( stickPos < midposition )                                         // Upper part of LOWER half 
    result = map(stickPos, LOWPOSITION, midposition, LOWOUT , NORMALOUT);
    
  if (stickPos < LOWPOSITION)                                           // Lowest part of LOWER half
    result = map(stickPos, MINPOSITION, LOWPOSITION, MINOUT, LOWOUT);

  return (float)result/MAXOUT;
}





// ------------------------------------------------------------------------------
