#include <muses_72323.h>
#include <SPI.h>


// GPIOs
#define MOSI  12  // DATA
#define MISO  13  // unused
#define SCK   2   // CLOCK
#define SS    0   // LATCH

// Four Muses chip adresses
#define ONE2MIX 0
#define ONE2CUE 1
#define TWO2MIX 2
#define TWO2CUE 3


static const SPISettings s_muses_spi_settings(250000, MSBFIRST, SPI_MODE0);
muses_72323_command_t command;


bool link_channels  = false;
bool use_soft_step  = true;
bool use_zero_cross = true;
muses_72323_zero_window_t zero_window = 0;
muses_72323_soft_step_clock_divider_t clock_divider = 0;
muses_72323_soft_step_clock_source_t  soft_step_clock = MUSES_72323_SOFT_STEP_CLOCK_INTERNAL;  



// ---------------------------------------------------------------------------------
// Function declarations
void volume	     (uint8_t chip_address, uint16_t volume_left, uint16_t volume_right);
void attenuation (uint8_t chip_address, uint8_t channel, uint16_t attenuation);
void mute        (uint8_t chip_address, uint8_t channel);
void gain        (uint8_t chip_address, uint8_t gain_left, uint8_t gain_right);

void musesBegin(void);
void musesConfigure(uint8_t chip_address);
void musesSend(muses_72323_command_t command);






// ---------------------------------------------------------------------------------
// Volume: 0.00 to 1.00  
void volume	(uint8_t chip_address, float volume_left, float volume_right) {
	attenuation (chip_address, 0, (uint16_t)(MUSES_72323_MAX_ATTENUATION * (1.0-volume_left))); 
	attenuation (chip_address, 1, (uint16_t)(MUSES_72323_MAX_ATTENUATION * (1.0-volume_right))); 
}


// ---------------------------------------------------------------------------------
// Attenuation: 0 to 447 (447 = full attenuation = silent)
void attenuation (uint8_t chip_address, uint8_t channel, uint16_t attenuation) {
  attenuation = constrain ( attenuation, 0, MUSES_72323_MAX_ATTENUATION);
  muses_72323_set_volume(&command, (muses_72323_chip_address_t)chip_address, (muses_72323_channel_t)channel, (muses_72323_attenuation_t)attenuation, use_soft_step);
  musesSend(command);
//  Serial.printf("chip %d \t channel %d \t attenuation %d \n", chip_address, channel, attenuation);
}


// ---------------------------------------------------------------------------------
// GAIN: 0 to 7
void gain (uint8_t chip_address, uint8_t gain_left, uint8_t gain_right) {
  gain_left  = constrain ( gain_left,  0, MUSES_72323_MAX_CHANNEL_GAIN);
  gain_right = constrain ( gain_right, 0, MUSES_72323_MAX_CHANNEL_GAIN);
  muses_72323_set_gain(&command, (muses_72323_chip_address_t)chip_address, (muses_72323_channel_gain_t)gain_left, (muses_72323_channel_gain_t)gain_right, link_channels, use_zero_cross);
  musesSend(command);
}


// ---------------------------------------------------------------------------------
void mute (uint8_t chip_address, uint8_t channel) {
  muses_72323_mute(&command, (muses_72323_chip_address_t)chip_address, (muses_72323_channel_t)channel, use_zero_cross);
  musesSend(command);
}


// ---------------------------------------------------------------------------------
void musesBegin(void) {
  pinMode(SS, OUTPUT);
  digitalWrite(SS, HIGH);  
  SPI.begin(SCK, MISO, MOSI);
  musesConfigure(ONE2MIX);
  musesConfigure(TWO2MIX);
  musesConfigure(ONE2CUE);
  musesConfigure(TWO2CUE);
}


// ---------------------------------------------------------------------------------
void musesConfigure(uint8_t chip_address) {
  muses_72323_configure(&command, (muses_72323_chip_address_t)chip_address, zero_window, clock_divider, soft_step_clock);
  musesSend(command);
}


// ---------------------------------------------------------------------------------
void musesSend(muses_72323_command_t command) {
/* 
  Serial.print(highByte(command),BIN);
  Serial.print("\t");
  Serial.println(lowByte(command),BIN);
*/
  SPI.beginTransaction(s_muses_spi_settings);  
  digitalWrite(SS, LOW);
  SPI.transfer(highByte(command));
  SPI.transfer(lowByte(command));  
  digitalWrite(SS, HIGH);
  SPI.endTransaction();  
  delay(5);
}





// ---------------------- the end ----------------------------
