#include <Arduino.h>
#include "OneButton.h"
#include <muses.h>
#include <rotaries.h>
#include <2Dxfader.h>



#define STARTUPVOLUME 0.9
float one2mix = STARTUPVOLUME;
float two2mix = STARTUPVOLUME;
float one2cue = STARTUPVOLUME;
float two2cue = STARTUPVOLUME;


#define CUEMODE_MAINMIX   0
#define CUEMODE_AUTO      1
#define CUEMODE_CUEFADER  2
#define MAXCUEMODES       3
uint8_t cuemode = CUEMODE_AUTO;


OneButton buttonONE(rotONE_SW, true);
OneButton buttonTWO(rotTWO_SW, true);
OneButton buttonMIX(rotMIX_SW, true);
OneButton buttonCUE(rotCUE_SW, true);

void clickONE (void) {
  
}
void clickTWO (void) {
  
}
void clickMIX (void) {
  
}
void clickCUE (void) {
  if (cuemode) cuemode = 0;
  else cuemode = 1;
}
void doubleclickCUE (void) {
  cuemode = 0; 
  Serial.println(cuemode);
}



// ------------------------------------------------------------------------------------
void setup() {
  Serial.begin(115200);
  delay(200);
  Serial.println("Booting...");
  rotariesBegin();
  musesBegin();

  buttonONE.attachClick(clickONE);
  buttonTWO.attachClick(clickTWO);
  buttonMIX.attachClick(clickMIX);
  buttonCUE.attachClick(clickCUE);
  buttonCUE.attachDoubleClick(doubleclickCUE);

}


void loop() {  
  
  readrotaries();  
  readfaders();
  setmixvolumes();
  setcuevolumes();
  buttonONE.tick();
  buttonTWO.tick();
  buttonMIX.tick();
  buttonCUE.tick();

  unsigned long loopnow = millis();
  static unsigned long timerone;
  if (loopnow - timerone >= 250) {
    timerone = loopnow;
  }
}



// ------------------------------------------------------------------------------------
void readrotaries(void) {

  const int16_t minvol = 0;
  const int16_t maxvol = 1000;
  const int16_t mastervolumestep = 8;
  const int16_t inputvolumestep = 4;
  
  int16_t moved;

  // Gain INPUT ONE
  if(moved = rotaryPosition(rotONE)) {                      
    one2mix = (float)(constrain((one2mix*maxvol)+(moved*inputvolumestep), minvol, maxvol))/maxvol; 
    one2cue = (float)(constrain((one2cue*maxvol)+(moved*inputvolumestep), minvol, maxvol))/maxvol; 
  }
  
  // Gain INPUT TWO
  if(moved = rotaryPosition(rotTWO)) {                      
    two2mix = (float)(constrain((two2mix*maxvol)+(moved*inputvolumestep), minvol, maxvol))/maxvol; 
    two2cue = (float)(constrain((two2cue*maxvol)+(moved*inputvolumestep), minvol, maxvol))/maxvol; 
  }
  
  // Volume MIX OUTPUT 
  if(moved = rotaryPosition(rotMIX)) {                      
    if (one2mix) one2mix = (float)(constrain((one2mix*maxvol)+(moved*mastervolumestep), minvol, maxvol))/maxvol;  
    if (two2mix) two2mix = (float)(constrain((two2mix*maxvol)+(moved*mastervolumestep), minvol, maxvol))/maxvol; 
  }
  
  // Volume CUE OUTPUT
  if(moved = rotaryPosition(rotCUE)) {                      
    if (one2cue) one2cue = (float)(constrain((one2cue*maxvol)+(moved*mastervolumestep), minvol, maxvol))/maxvol;  
    if (two2cue) two2cue = (float)(constrain((two2cue*maxvol)+(moved*mastervolumestep), minvol, maxvol))/maxvol; 
  }
}




// ------------------------------------------------------------------------------------
void setmixvolumes(void) {

  float one2mixleft;
  float one2mixright;
  float two2mixleft;
  float two2mixright;

  // SOURCE ONE TO MIX
  one2mixleft  = xfade(joy1, 1.0-joy2) * one2mix;
  one2mixright = xfade(joy2, 1.0-joy1) * one2mix;
  volume(ONE2MIX, one2mixleft, one2mixright);
  
  // SOURCE TWO TO MIX
  two2mixleft  = xfade(1.0-joy1, joy2) * two2mix;
  two2mixright = xfade(1.0-joy2, joy1) * two2mix;
  volume(TWO2MIX, two2mixleft, two2mixright);

//  Serial.printf("%f\t %f\t %f\t %f\n", joy1, joy2, cuex, curv);
//  Serial.printf("%f\t %f\t %f\t %f\n", one2mix, two2mix, one2cue, two2cue);
//  Serial.printf("0 %f\t %f\t %f\t %f\n", one2mixleft*100, one2mixright*100, two2mixleft*100);
}


// ------------------------------------------------------------------------------------
void setcuevolumes(void) {

  float one2cueleft;
  float one2cueright;
  float two2cueleft;
  float two2cueright;

  static float prevcuex = cuex;
  if ( cuex > prevcuex*1.1 || cuex < prevcuex*0.9) {
    cuemode = CUEMODE_CUEFADER;
    prevcuex = cuex;
  }
  
  if (cuemode == CUEMODE_AUTO) {
    one2cueleft  = xfade(1.0-joy1, joy2) * one2cue;
    one2cueright = xfade(1.0-joy2, joy1) * one2cue;
    two2cueleft  = xfade(joy1, 1.0-joy2) * two2cue;
    two2cueright = xfade(joy2, 1.0-joy1) * two2cue;
  }
  if (cuemode == CUEMODE_CUEFADER) {
    one2cueleft  = xfade(1.0-cuex, 1.0-cuex) * one2cue;
    one2cueright = xfade(1.0-cuex, 1.0-cuex) * one2cue;
    two2cueleft  = xfade(cuex, cuex) * two2cue;
    two2cueright = xfade(cuex,  cuex) * two2cue;
  }
  if (cuemode == CUEMODE_MAINMIX) {
    one2cueleft  = xfade(joy1, 1.0-joy2) * one2cue;
    one2cueright = xfade(joy2, 1.0-joy1) * one2cue;
    two2cueleft  = xfade(1.0-joy1, joy2) * two2cue;
    two2cueright = xfade(1.0-joy2, joy1) * two2cue;
  }
  volume(ONE2CUE, one2cueleft, one2cueright);
  volume(TWO2CUE, two2cueleft, two2cueright);


//  Serial.printf("%f\t %f\t %f\t %f\n", joy1, joy2, cuex, curv);
//  Serial.printf("%f\t %f\t %f\t %f\n", one2mix, two2mix, one2cue, two2cue);
//  Serial.printf("0 %f\t %f\t %f\t %f\n", one2cueleft*100, one2cueright*100, two2cueleft*100, two2cueright*100);
}




// ------------------------------------------------------------------------------------
