#include <RotaryEncoder.h>


// GPIOs
#define rotONE_CL 17
#define rotONE_DT 16
#define rotONE_SW 21

#define rotTWO_CL 27
#define rotTWO_DT 25
#define rotTWO_SW 22

#define rotMIX_CL 18
#define rotMIX_DT 26
#define rotMIX_SW 4

#define rotCUE_CL 23
#define rotCUE_DT 19
#define rotCUE_SW 14

RotaryEncoder *rotONE = nullptr;
RotaryEncoder *rotTWO = nullptr;
RotaryEncoder *rotMIX = nullptr;
RotaryEncoder *rotCUE = nullptr;



IRAM_ATTR void handleRotateONE() { 
  rotONE->tick(); 
}
IRAM_ATTR void handleRotateTWO() { 
  rotTWO->tick(); 
}
IRAM_ATTR void handleRotateMIX() { 
  rotMIX->tick(); 
}
IRAM_ATTR void handleRotateCUE() { 
  rotCUE->tick(); 
}



// ------------------------------------------------------------------------------------
int16_t rotaryPosition(RotaryEncoder *rotary) {
  int16_t moved = rotary->getPosition();
  rotary->setPosition(0);
  return moved;
}


// ------------------------------------------------------------------------------------
void rotariesBegin(void) {
  rotONE = new RotaryEncoder(rotONE_CL, rotONE_DT, RotaryEncoder::LatchMode::TWO03);  //  FOUR3 FOUR0  TWO03
  attachInterrupt(digitalPinToInterrupt(rotONE_CL), handleRotateONE, CHANGE);
  attachInterrupt(digitalPinToInterrupt(rotONE_DT), handleRotateONE, CHANGE);
  rotTWO = new RotaryEncoder(rotTWO_CL, rotTWO_DT, RotaryEncoder::LatchMode::TWO03);  //  FOUR3 FOUR0  TWO03
  attachInterrupt(digitalPinToInterrupt(rotTWO_CL), handleRotateTWO, CHANGE);
  attachInterrupt(digitalPinToInterrupt(rotTWO_DT), handleRotateTWO, CHANGE);
  rotMIX = new RotaryEncoder(rotMIX_CL, rotMIX_DT, RotaryEncoder::LatchMode::TWO03);  //  FOUR3 FOUR0  TWO03
  attachInterrupt(digitalPinToInterrupt(rotMIX_CL), handleRotateMIX, CHANGE);
  attachInterrupt(digitalPinToInterrupt(rotMIX_DT), handleRotateMIX, CHANGE);
  rotCUE = new RotaryEncoder(rotCUE_CL, rotCUE_DT, RotaryEncoder::LatchMode::TWO03);  //  FOUR3 FOUR0  TWO03
  attachInterrupt(digitalPinToInterrupt(rotCUE_CL), handleRotateCUE, CHANGE);
  attachInterrupt(digitalPinToInterrupt(rotCUE_DT), handleRotateCUE, CHANGE);
}
